from __future__ import absolute_import
from __future__ import division

from numpy import asarray
from . import cycler_ as cycler
from .util import consensus_curve

from limix_util.dict import OrderedDict

class ConsensusCurve(object):
    def __init__(self, axes):
        self._axes = axes
        self._x = OrderedDict()
        self._y = OrderedDict()
        self._color = dict()

    def add(self, x, y, label, color=None):

        if label not in self._x:
            self._x[label] = []
            self._y[label] = []

            if color:
                self._color[label] = color
            else:
                prop = cycler.next(self._axes)
                if 'color' in prop:
                    self._color[label] = prop['color']
                else:
                    self._color[label] = None

        self._x[label].append(asarray(x))
        self._y[label].append(asarray(y))

        self._axes.tick_params(axis='both', which='major', direction='out')
        self._axes.spines['right'].set_visible(False)
        self._axes.spines['top'].set_visible(False)
        self._axes.yaxis.set_ticks_position('left')
        self._axes.xaxis.set_ticks_position('bottom')

    def plot(self, same_steps=False):
        axes = self._axes

        labels = self._x.keys()
        for label in labels:
            self._plot_for_label(label, same_steps)

        return axes

    def _plot_for_label(self, label, same_steps):
        axes = self._axes

        xavg, ybottom, yavg, ytop = consensus_curve(self._x[label],
                                                    self._y[label],
                                                    same_steps=same_steps)

        color = self._color[label]
        axes.plot(xavg, yavg, color=color)
        axes.fill_between(xavg, ybottom, ytop, lw=0, edgecolor='None',
                          facecolor=color, alpha=0.1, interpolate=True)
        axes.fill_between(xavg, ybottom, ytop, lw=0.05, edgecolor=color,
                          facecolor='None', interpolate=True)
        # axes.plot(xavg, ytop, lw=0.2, color=color)
        # axes.plot(xavg, ybottom, lw=0.2, color=color)
