from matplotlib.transforms import BboxTransformTo
from matplotlib.transforms import Bbox
from limix_util.dict import traverse_dict
from math import sqrt

def _grid_trans(nrow, ncol, i):
    r = int(i / ncol)
    c = int(i % ncol)
    h = 1. / nrow
    w = 1. / ncol
    y0 = 1 - (r+1) * h
    x0 = c * w
    box = Bbox.from_bounds(x0, y0, w, h)
    return BboxTransformTo(box)

class _GridPlot(object):
    def __init__(self, visit_func, fig):
        self._fig = fig

        self._min_box_width = float('inf')
        self._min_box_height = float('inf')
        self._visit_func = visit_func

        h = 0.1 * 2/4.
        self._title_trans = BboxTransformTo(Bbox.from_bounds(0, 1-h, 1, h))
        self._body_trans = BboxTransformTo(Bbox.from_bounds(0, 0, 1, 1-h))
        self._axes_trans = BboxTransformTo(Bbox.from_bounds(0.1, 0.15, 0.8, 0.75))

        # self._color_cycle = cycle(color_palette("husl", 8))

    def traverse(self, group_names, data):
        trans = BboxTransformTo(Bbox.from_bounds(0.015, 0.005, 1-0.03, 1-0.005))
        opts = dict(names=group_names, trans=trans)
        traverse_dict(data, self._callback, opts)

    def adjust_figsize(self):
        # default_fig_size
        dfs = (8., 6.)
        # default_axis_size
        das = (0.77500000000000002, 0.80000000000000004)

        width = dfs[0] * das[0] / self._min_box_width
        height = dfs[1] * das[1] / self._min_box_height
        self._fig.set_size_inches(width, height)


    def _callback(self, d, opt):
        n = len(d)
        # plot_this = n > 0 and\
        #             (not isinstance(d, dict) or len(opt['names']) == 0)
        # import ipdb; ipdb.set_trace()
        if opt is None:
            return
        plot_this = n > 0 and len(opt['names']) == 0

        if plot_this:
            trans = self._axes_trans + opt['trans']
            (x0, y0) = trans.transform([0, 0])
            (x1, y1) = trans.transform([1, 1])
            axes = self._fig.add_axes([x0, y0, x1-x0, y1-y0])

            self._visit_func(d, axes)

            box = axes.get_position()
            self._min_box_width = min(box.width, self._min_box_width)
            self._min_box_height = min(box.height, self._min_box_height)
        else:
            (x, y) = (self._title_trans + opt['trans']).transform((0.5, 0.5))

            # color = next(self._color_cycle)
            self._fig.text(x, y, opt['names'][0], horizontalalignment='center',
                           verticalalignment='center')


            nrow = ncol = int(sqrt(n))
            if nrow * ncol < n:
                ncol += 1
            if nrow * ncol < n:
                nrow += 1

            opts = []

            for i in xrange(len(d)):
                grid_trans = _grid_trans(nrow, ncol, i)
                cell_trans = (grid_trans + self._body_trans + opt['trans'])
                (x, y) = (self._title_trans + cell_trans).transform((0.5, 1.))

                self._fig.text(x, y, d.keys()[i], horizontalalignment='center',
                               verticalalignment='center')

                nopt = dict(names=opt['names'][1:],
                            trans=self._body_trans+cell_trans)
                opts.append(nopt)

            return opts

def grid_plot(group_names, data, visit_func, fig):
    gp = _GridPlot(visit_func, fig=fig)
    gp.traverse(group_names, data)
    gp.adjust_figsize()
