from __future__ import division
from collections import OrderedDict
from numpy import asarray
from matplotlib.transforms import blended_transform_factory
from matplotlib.artist import setp
from limix_util.set import OrderedSet
from . import cycler_ as cycler

class HistPlot(object):
    def __init__(self, axes):
        self._axes = axes
        self._values = dict()
        self._colors = dict()
        self.legend = True

    def _labels(self):
        return list(self._values.keys())

    # @property
    # def _group_labels(self):
    #     return sorted(self._values.keys())
    #
    # @property
    # def _unique_labels(self):
    #     labels = OrderedSet()
    #     for grp in self._group_labels:
    #         for l in self._labels(grp):
    #             labels.add(l)
    #     return list(labels)

    def get_color(self, label):
        return self._colors[label]

    def set_color(self, label, color):
        self._colors[label] = color

    def _get_values(self, label):
        vs = self._values[label]
        return vs

    def add(self, values, label, color=None, **props):
        self._values[label] = asarray(values).ravel()

        if label not in self._colors:
            if color:
                self._colors[label] = color
            else:
                prop = cycler.next(self._axes)
                if 'color' in prop:
                    self._colors[label] = prop['color']
                else:
                    self._colors[label] = None

    def plot(self):
        axes = self._axes

        width = 0.5
        pos = []
        left = 0
        data = []
        colors = []

        for l in self._labels():
            v = self._get_values(l)
            axes.hist(v, color=self.get_color(l))

        if self.legend:
            fakes = []
            for l in self._labels():
                fake, = axes.plot([1,1], color=self.get_color(l), lw=3)
                fakes.append(fake)

            labels = self._unique_labels
            axes.legend(fakes, labels, bbox_to_anchor=(0., 1.02, 1., .102), loc=3,
                    ncol=min(len(labels), 4), mode="expand", borderaxespad=0.,
                    frameon=False)

            for fake in fakes:
                fake.set_visible(False)

        # axes.xaxis.set_label_coords(0.5, -0.08)
        # axes.set_xticklabels([''] * len(axes.get_xticklabels()))
        axes.yaxis.set_ticks_position('left')
        axes.xaxis.set_ticks_position('none')

        axes.spines['right'].set_visible(False)
        axes.spines['top'].set_visible(False)

        return axes
